package com.heyang;

import java.util.List;
import java.util.Stack;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * JSOn tree builder
 * @author heyang
 *
 * 2020518
 */
public class Builder {
	private Node root;
	private int index;
	private List<Token> tokens;
	
	public Builder(List<Token> tokens) {
		this.tokens=tokens;
		this.index=1;
		
		this.root=new Node(null,Node.Type_List);
		addSubNode2(this.root);
	}

	/**
	 * Add branch/leaf to parent node
	 * @param parent
	 */
	private void addSubNode2(Node parent) {
		if(parent==null) {
			return;
		}
		
		Stack<Token> stack=new Stack<Token>();
		
		while(index<this.tokens.size()) {
			Token token=tokens.get(index);
			
			if(token.getType()==Token.TYPE_OPEN_BRACE) {// {
				Node newBraceNode=new Node(null,Node.Type_List);
				
				if(stack.size()>=2) {
					Token colonToken=stack.pop();
					Token keyToken=stack.pop();
					
					if(colonToken.getType()==Token.TYPE_COLON && keyToken.getType()==Token.TYPE_TEXT) {
						newBraceNode.setKey(keyToken.getText());
					}
				}
				
				parent.addChild(newBraceNode);
				
				index++;
				addSubNode2(newBraceNode);
			}else if(token.getType()==Token.TYPE_CLOSE_BRACE) { // }
				String text=getTextInStack(stack);
				
				if(text.length()>0) {
					final String keyValuePattern="(\"([_a-zA-Z]+[_a-zA-Z0-9]*)\")[:]([^,}]+)";
					
					if(Pattern.matches(keyValuePattern,text)) {
						java.util.regex.Pattern pattern=Pattern.compile(keyValuePattern);
				        Matcher matcher=pattern.matcher(text);
				        while(matcher.find()) {
				        	Node txt=new Node(matcher.group(1),matcher.group(3));
				        	parent.addChild(txt);
				        }
					}
				}
				
				stack.clear();
				index++;
				
				addSubNode2(parent.getParent());
			}else if(token.getType()==Token.TYPE_OPEN_BRACKET) { // [
				Node newBracketNode=new Node(null,Node.Type_Array);
				
				if(stack.size()>=2) {
					Token left1=stack.pop();
					Token left2=stack.pop();
					
					if(left1.getType()==Token.TYPE_COLON && left2.getType()==Token.TYPE_TEXT) {
						newBracketNode.setKey(left2.getText());
					}
				}
				
				parent.addChild(newBracketNode);
				
				index++;
				addSubNode2(newBracketNode);
			}else if(token.getType()==Token.TYPE_CLOSE_BRACKET) { // ]
				String text=getTextInStack(stack);
				
				if(text.length()>0) {
					final String keyValuePattern="(\"([_a-zA-Z]+[_a-zA-Z0-9]*)\")[:]([^,}]+)";
					
					if(Pattern.matches(keyValuePattern,text)) {
						java.util.regex.Pattern pattern=Pattern.compile(keyValuePattern);
				        Matcher matcher=pattern.matcher(text);
				        while(matcher.find()) {
				        	Node txt=new Node(matcher.group(1),matcher.group(3));
				        	parent.addChild(txt);
				        }
					}else {
			        
						java.util.regex.Pattern pattern=Pattern.compile("([^,]+)");
						Matcher matcher=pattern.matcher(text);
				        while(matcher.find()) {
				        	Node txt=new Node(null,matcher.group(1));
				        	parent.addChild(txt);
				        }
					}
				}
				
				stack.clear();
				index++;
				addSubNode2(parent.getParent());
			}else if(token.getType()==Token.TYPE_COMMA) {
				String text=getTextInStack(stack);
				
				if(text.length()>0) {
					final String keyValuePattern="(\"([_a-zA-Z]+[_a-zA-Z0-9]*)\")[:]([^,}]+)";
					
					if(Pattern.matches(keyValuePattern,text)) {
						java.util.regex.Pattern pattern=Pattern.compile(keyValuePattern);
				        Matcher matcher=pattern.matcher(text);
				        while(matcher.find()) {
				        	Node txt=new Node(matcher.group(1),matcher.group(3));
				        	parent.addChild(txt);
				        }
					}else {
			        
						java.util.regex.Pattern pattern=Pattern.compile("([^,]+)");
						Matcher matcher=pattern.matcher(text);
				        while(matcher.find()) {
				        	Node txt=new Node(null,matcher.group(1));
				        	parent.addChild(txt);
				        }
					}
				}
				
				stack.clear();
				index++;
			}else {
				stack.push(token);
				index++;
	 		}
		}
	}
	
	private String getTextInStack(Stack<Token> stack) {
		StringBuilder sb=new StringBuilder();
		for(int i=0;i<stack.size();i++) {
			Token t=stack.elementAt(i);
			sb.append(t.getText());
		}
		
		return sb.toString();
	}
	
	public Node getRoot() {
		return root;
	}
}
