package com.heyang;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.heyang.util.BracketsBalanceChecker;
import com.heyang.util.CommonUtil;

public class EntryPoint {
	private final static Logger log = LoggerFactory.getLogger(EntryPoint.class);
	
	public static void main(String[] args){
		log.info("JsonAnalyzer started.");
		
		if(args.length<1) {
			log.error("Please set json file path in arguments.");
		}
		
		String filePath=args[0];
		log.info("Begin to read file:'{}'",filePath);

		try {
			// Read context from file
			String jsonTxt=CommonUtil.readTextFromFile(filePath);
			log.info("Raw json text=\n{}",jsonTxt);
			
			// Check brackets balance
			BracketsBalanceChecker bbc=new BracketsBalanceChecker();
			boolean balanced=bbc.isBracketsBalanced(jsonTxt);
			if(balanced) {
				log.info("The brackets in read content are balanced.");
			}
			
			// Parse json to tokens
			Lexer l=new Lexer(jsonTxt);
			log.info("Compact json text=\n{}",l.getCompactJsonTxt());
			
			// Build json node tree
            Builder b=new Builder(l.getTokenList());
            Node root=b.getRoot();
            log.info("Formatted json=\n{}",root.toString());
			
		}catch(Exception ex) {
			log.error(ex.getMessage());
		}finally {
			log.info("JsonAnalyzer end.");
		}
	}
}
