package com.heyang;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang.StringUtils;

/**
 * Parse json string to tokens
 * @author Heyang
 *
 */
public class Lexer {
	private List<Token> tokens;

	public Lexer(String jsonTxt) {
		tokens = new ArrayList<Token>();

		String bundle = "";
		for (int i = 0; i < jsonTxt.length(); i++) {
			char c = jsonTxt.charAt(i);

			if (Character.isWhitespace(c)) {
				continue;
			} else if (c == '{') {
				tokens.add(new Token(c, Token.TYPE_OPEN_BRACE));
			} else if (c == '}') {
				if (StringUtils.isNotEmpty(bundle)) {
					tokens.add(new Token(bundle, Token.TYPE_TEXT));
					bundle = "";
				}

				tokens.add(new Token(c, Token.TYPE_CLOSE_BRACE));
			} else if (c == '[') {
				tokens.add(new Token(c, Token.TYPE_OPEN_BRACKET));
			} else if (c == ']') {
				if (StringUtils.isNotEmpty(bundle)) {
					tokens.add(new Token(bundle, Token.TYPE_TEXT));
					bundle = "";
				}
				
				tokens.add( new Token(c, Token.TYPE_CLOSE_BRACKET));
			} else if (c == ',') {
				if (StringUtils.isNotEmpty(bundle)) {
					tokens.add(new Token(bundle, Token.TYPE_TEXT));
					bundle = "";
				}

				tokens.add(new Token(c, Token.TYPE_COMMA));
			} else if (c == ':') {
				if (StringUtils.isNotEmpty(bundle)) {
					tokens.add(new Token(bundle, Token.TYPE_TEXT));
					bundle = "";
				}

				tokens.add(new Token(c, Token.TYPE_COLON));
			} else {
				bundle += c;
			}
		}
	}

	public List<Token> getTokenList() {
		return tokens;
	}

	// Just for test
	public void printTokens() {
		int idx = 0;
		for (Token t : tokens) {
			idx++;
			System.out.println("#" + idx + " " + t.getText());
		}
	}
	
	public String getCompactJsonTxt() {
		StringBuilder sb=new StringBuilder();
		
		for (Token t : tokens) {
			sb.append(t.getText());
		}
		
		return sb.toString();
	}
}
